@@ -204,51 +204,23 @@ gdict_window_set_sidebar_visible (GdictWindow *window,
     }
 }
 
-static void
-gdict_window_set_statusbar_visible (GdictWindow *window,
-				    gboolean     is_visible)
-{
-  g_assert (GDICT_IS_WINDOW (window));
-
-  is_visible = !!is_visible;
-
-  if (is_visible != window->statusbar_visible)
-    {
-      GAction *action = g_action_map_lookup_action (G_ACTION_MAP (window),
-                                                    "view-statusbar");
-      g_action_change_state (action, g_variant_new_boolean (is_visible));
-    }
-}
-
 static void
 gdict_window_definition_cb (GdictContext    *context,
 			    GdictDefinition *definition,
 			    GdictWindow     *window)
 {
-  gint total, n;
-  gdouble fraction;
-
   g_assert (GDICT_IS_WINDOW (window));
 
-  total = gdict_definition_get_total (definition);
-  n = window->current_definition + 1;
-
-  fraction = CLAMP (((gdouble) n / (gdouble) total), 0.0, 1.0);
-
-  gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (window->progress),
-		  		 fraction);
   while (gtk_events_pending ())
     gtk_main_iteration ();
 
-  window->current_definition = n;
+  window->current_definition++;
 }
 
 static void
 gdict_window_lookup_start_cb (GdictContext *context,
 			      GdictWindow  *window)
 {
-  gchar *message;
-
   if (!window->word)
     return;
 
@@ -258,28 +230,21 @@ gdict_window_lookup_start_cb (GdictContext *context,
       window->busy_cursor = gdk_cursor_new_for_display (display, GDK_WATCH);
     }
 
-  message = g_strdup_printf (_("Searching for '%s'..."), window->word);
-  
-  if (window->status && window->statusbar_visible)
-    gtk_statusbar_push (GTK_STATUSBAR (window->status), 0, message);
-
-  if (window->progress)
-    gtk_widget_show (window->progress);
-
   window->max_definition = -1;
   window->last_definition = 0;
   window->current_definition = 0;
 
   gdk_window_set_cursor (gtk_widget_get_window (GTK_WIDGET (window)), window->busy_cursor);
 
-  g_free (message);
+  gtk_spinner_start (GTK_SPINNER (window->spinner));
+  gtk_widget_show (window->spinner);
+  gtk_stack_set_visible_child_name (GTK_STACK (window->stack), "spinner");
 }
 
 static void
 gdict_window_lookup_end_cb (GdictContext *context,
 			    GdictWindow  *window)
 {
-  gchar *message;
   gint count;
   GtkTreeIter iter;
   GdictSource *source;
@@ -289,20 +254,6 @@ gdict_window_lookup_end_cb (GdictContext *context,
 
   window->max_definition = count - 1;
 
-  if (count == 0)
-    message = g_strdup (_("No definitions found"));
-  else 
-    message = g_strdup_printf (ngettext("A definition found",
-					"%d definitions found",
-					count),
-		    	       count);
-
-  if (window->status && window->statusbar_visible)
-    gtk_statusbar_push (GTK_STATUSBAR (window->status), 0, message);
-
-  if (window->progress)
-    gtk_widget_hide (window->progress);
-
   /* we clone the context, so that the signals that it
    * fires do not get caught by the signal handlers we
    * use for getting the definitions.
@@ -327,7 +278,10 @@ gdict_window_lookup_end_cb (GdictContext *context,
     }
 
   gdk_window_set_cursor (gtk_widget_get_window (GTK_WIDGET (window)), NULL);
-  g_free (message);
+
+  gtk_stack_set_visible_child_name (GTK_STACK (window->stack), "main");
+  gtk_spinner_stop (GTK_SPINNER (window->spinner));
+  gtk_widget_hide (window->spinner);
 
   if (count == 0)
     {
@@ -344,13 +298,11 @@ gdict_window_error_cb (GdictContext *context,
 		       GdictWindow  *window)
 {
   gdk_window_set_cursor (gtk_widget_get_window (GTK_WIDGET (window)), NULL);
+
+  gtk_stack_set_visible_child_name (GTK_STACK (window->stack), "main");
+  gtk_spinner_stop (GTK_SPINNER (window->spinner));
+  gtk_widget_hide (window->spinner);
   
-  if (window->status && window->statusbar_visible)
-    gtk_statusbar_push (GTK_STATUSBAR (window->status), 0,
-                        _("No definitions found"));
-
-  gtk_widget_hide (window->progress);
-
   /* launch the speller only on NO_MATCH */
   if (error->code == GDICT_CONTEXT_ERROR_NO_MATCH)
     {
@@ -756,7 +708,6 @@ gdict_window_store_state (GdictWindow *window)
   g_key_file_set_integer (state_key, "WindowState", "Height", window->current_height);
   g_key_file_set_boolean (state_key, "WindowState", "IsMaximized", window->is_maximized);
   g_key_file_set_boolean (state_key, "WindowState", "SidebarVisible", window->sidebar_visible);
-  g_key_file_set_boolean (state_key, "WindowState", "StatusbarVisible", window->statusbar_visible);
   g_key_file_set_integer (state_key, "WindowState", "SidebarWidth", window->sidebar_width);
 
   page_id = gdict_sidebar_current_page (GDICT_SIDEBAR (window->sidebar));
@@ -841,13 +792,6 @@ gdict_window_load_state (GdictWindow *window)
       window->sidebar_visible = FALSE;
     }
 
-  window->statusbar_visible = g_key_file_get_boolean (state_key, "WindowState", "StatusbarVisible", &error);
-  if (error != NULL)
-    {
-      g_clear_error (&error);
-      window->statusbar_visible = FALSE;
-    }
-
   window->sidebar_width = g_key_file_get_integer (state_key, "WindowState", "SidebarWidth", &error);
   if (error != NULL)
     {
@@ -866,43 +810,6 @@ gdict_window_load_state (GdictWindow *window)
   g_free (state_file);
 }
 
-static void
-gdict_window_cmd_file_new (GSimpleAction   *action,
-                           GVariant        *parameter,
-                           gpointer         user_data)
-{
-  GdictWindow *window = user_data;
-  GtkWidget *new_window;
-  gchar *word = NULL;
-
-  gdict_window_store_state (window);
-
-  word = gdict_defbox_get_selected_word (GDICT_DEFBOX (window->defbox));
-  if (word)
-    {
-      new_window = gdict_window_new (GDICT_WINDOW_ACTION_LOOKUP,
-                                     window->loader,
-                                     NULL,
-                                     NULL,
-                                     NULL,
-                                     word);
-      g_free (word);
-    }
-  else
-    new_window = gdict_window_new (GDICT_WINDOW_ACTION_CLEAR,
-                                   window->loader,
-                                   NULL,
-                                   NULL,
-                                   NULL,
-                                   NULL);
-
-  gtk_window_set_application (GTK_WINDOW (new_window),
-                              gtk_window_get_application (GTK_WINDOW (window)));
-  gtk_widget_show (new_window);
-  
-  g_signal_emit (window, gdict_window_signals[CREATED], 0, new_window);
-}
-
 static void
 gdict_window_cmd_save_as (GSimpleAction   *action,
                           GVariant        *parameter,
@@ -1091,25 +998,6 @@ gdict_window_cmd_change_view_sidebar (GSimpleAction *action,
   g_simple_action_set_state (action, state);
 }
 
-static void
-gdict_window_cmd_change_view_statusbar (GSimpleAction *action,
-                                        GVariant      *state,
-                                        gpointer       user_data)
-{
-  GdictWindow *window = user_data;
-
-  g_assert (GDICT_IS_WINDOW (window));
-
-  window->statusbar_visible = g_variant_get_boolean (state);
-
-  if (window->statusbar_visible)
-    gtk_widget_show (window->status);
-  else
-    gtk_widget_hide (window->status);
-
-  g_simple_action_set_state (action, state);
-}
-
 static void
 gdict_window_cmd_view_speller (GSimpleAction *action,
                                GVariant      *parameter,
@@ -1273,7 +1161,6 @@ gdict_window_cmd_escape (GSimpleAction *action,
 static const GActionEntry entries[] =
 {
   /* File menu */
-  { "new", gdict_window_cmd_file_new, NULL, NULL, NULL },
   { "save-as", gdict_window_cmd_save_as, NULL, NULL, NULL },
   { "preview", gdict_window_cmd_file_preview, NULL, NULL, NULL },
   { "print", gdict_window_cmd_file_print, NULL, NULL, NULL },
@@ -1295,8 +1182,6 @@ static const GActionEntry entries[] =
   /* View menu */
   { "view-sidebar", activate_toggle, NULL, "false",
     gdict_window_cmd_change_view_sidebar },
-  { "view-statusbar", activate_toggle, NULL, "false",
-    gdict_window_cmd_change_view_statusbar },
   { "view-speller", gdict_window_cmd_view_speller, NULL, NULL, NULL },
   { "view-source", gdict_window_cmd_view_sources, NULL, NULL, NULL },
   { "view-db", gdict_window_cmd_view_databases, NULL, NULL, NULL },
@@ -1363,16 +1248,6 @@ source_activated_cb (GdictSourceChooser *chooser,
   g_signal_handlers_block_by_func (chooser, source_activated_cb, window);
   gdict_window_set_source_name (window, source_name);
   g_signal_handlers_unblock_by_func (chooser, source_activated_cb, window);
-
-  if (window->status && window->statusbar_visible)
-    {
-      gchar *message;
-
-      message = g_strdup_printf (_("Dictionary source `%s' selected"),
-                                 gdict_source_get_description (source));
-      gtk_statusbar_push (GTK_STATUSBAR (window->status), 0, message);
-      g_free (message);
-    }
 }
 
 static void
@@ -1384,15 +1259,6 @@ strategy_activated_cb (GdictStrategyChooser *chooser,
   g_signal_handlers_block_by_func (chooser, strategy_activated_cb, window);
   gdict_window_set_strategy (window, strat_name);
   g_signal_handlers_unblock_by_func (chooser, strategy_activated_cb, window);
-
-  if (window->status && window->statusbar_visible)
-    {
-      gchar *message;
-
-      message = g_strdup_printf (_("Strategy `%s' selected"), strat_desc);
-      gtk_statusbar_push (GTK_STATUSBAR (window->status), 0, message);
-      g_free (message);
-    }
 }
 
 static void
@@ -1404,15 +1270,6 @@ database_activated_cb (GdictDatabaseChooser *chooser,
   g_signal_handlers_block_by_func (chooser, database_activated_cb, window);
   gdict_window_set_database (window, db_name);
   g_signal_handlers_unblock_by_func (chooser, database_activated_cb, window);
-
-  if (window->status && window->statusbar_visible)
-    {
-      gchar *message;
-
-      message = g_strdup_printf (_("Database `%s' selected"), db_desc);
-      gtk_statusbar_push (GTK_STATUSBAR (window->status), 0, message);
-      g_free (message);
-    }
 }
 
 static void
@@ -1424,15 +1281,6 @@ speller_word_activated_cb (GdictSpeller *speller,
   gtk_entry_set_text (GTK_ENTRY (window->entry), word);
   
   gdict_window_set_word (window, word, db_name);
-
-  if (window->status && window->statusbar_visible)
-    {
-      gchar *message;
-
-      message = g_strdup_printf (_("Word `%s' selected"), word);
-      gtk_statusbar_push (GTK_STATUSBAR (window->status), 0, message);
-      g_free (message);
-    }
 }
 
 static void
@@ -1482,9 +1330,6 @@ sidebar_page_changed_cb (GdictSidebar *sidebar,
       message = NULL;
       break;
     }
-
-  if (message && window->status && window->statusbar_visible)
-    gtk_statusbar_push (GTK_STATUSBAR (window->status), 0, message);
 }
 
 static void
@@ -1500,17 +1345,17 @@ gdict_window_link_clicked (GdictDefbox *defbox,
                            GdictWindow *window)
 {
   GtkWidget *new_window;
+  GtkApplication *application = gtk_window_get_application (GTK_WINDOW (window));
 
   gdict_window_store_state (window);
 
   new_window = gdict_window_new (GDICT_WINDOW_ACTION_LOOKUP,
+                                 application,
                                  window->loader,
                                  NULL,
                                  NULL,
                                  NULL,
                                  link_text);
-  gtk_window_set_application (GTK_WINDOW (new_window),
-                              gtk_window_get_application (GTK_WINDOW (window)));
   gtk_widget_show (new_window);
   
   g_signal_emit (window, gdict_window_signals[CREATED], 0, new_window);
@@ -1598,10 +1443,9 @@ gdict_window_constructor (GType                  type,
 {
   GObject *object;
   GdictWindow *window;
-  GtkWidget *hbox;
+  GtkBuilder *builder;
   GtkWidget *handle;
   GtkWidget *frame1, *frame2;
-  GtkWidget *vbox;
   GtkWidget *button;
   PangoFontDescription *font_desc;
   gchar *font_name;
@@ -1615,30 +1459,20 @@ gdict_window_constructor (GType                  type,
   /* recover the state */
   gdict_window_load_state (window);
 
-  window->main_box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
-  gtk_container_add (GTK_CONTAINER (window), window->main_box);
-  gtk_widget_show (window->main_box);
-  
   /* build menus */
   g_action_map_add_action_entries (G_ACTION_MAP (window),
                                    entries, G_N_ELEMENTS (entries),
                                    window);
   gdict_window_ensure_menu_state (window);
 
-  vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
-  gtk_container_set_border_width (GTK_CONTAINER (vbox), 6);
-  gtk_container_add (GTK_CONTAINER (window->main_box), vbox);
-  gtk_widget_show (vbox);
-  
-  hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 12);
-  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
-  gtk_widget_show (hbox);
-  
-  button = gtk_button_new_with_mnemonic (_("Look _up"));
-  g_signal_connect_swapped (button, "clicked",
-                            G_CALLBACK (lookup_word),
-                            window);
-  gtk_box_pack_end (GTK_BOX (hbox), button, FALSE, FALSE, 0);
+  button = gtk_menu_button_new ();
+  builder = gtk_builder_new ();
+  gtk_builder_add_from_resource (builder, "/org/gnome/Dictionary/gdict-app-menus.ui", NULL);
+  gtk_menu_button_set_menu_model (GTK_MENU_BUTTON (button),
+                                  G_MENU_MODEL (gtk_builder_get_object (builder, "menubar")));
+  g_object_unref (builder);
+  gtk_menu_button_set_direction (GTK_MENU_BUTTON (button), GTK_ARROW_NONE);
+  gtk_header_bar_pack_end (GTK_HEADER_BAR (window->header_bar), button);
   gtk_widget_show (button);
 
   window->completion_model = gtk_list_store_new (COMPLETION_N_COLUMNS,
@@ -1651,7 +1485,6 @@ gdict_window_constructor (GType                  type,
   gtk_entry_completion_set_text_column (window->completion,
 		  			COMPLETION_TEXT_COLUMN);
   
-  window->entry = gtk_entry_new ();
   if (window->word)
     gtk_entry_set_text (GTK_ENTRY (window->entry), window->word);
   
@@ -1660,11 +1493,9 @@ gdict_window_constructor (GType                  type,
   g_signal_connect_swapped (window->entry, "activate",
                             G_CALLBACK (lookup_word),
                             window);
-  gtk_box_pack_start (GTK_BOX (hbox), window->entry, TRUE, TRUE, 0);
-  gtk_widget_show (window->entry);
 
   handle = gtk_paned_new (GTK_ORIENTATION_HORIZONTAL);
-  gtk_box_pack_start (GTK_BOX (vbox), handle, TRUE, TRUE, 0);
+  gtk_box_pack_end (GTK_BOX (window->main_box), handle, TRUE, TRUE, 0);
   gtk_widget_show (handle);
 
   frame1 = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
@@ -1785,18 +1616,6 @@ gdict_window_constructor (GType                  type,
       g_action_change_state (action, g_variant_new_boolean (TRUE));
     }
 
-  window->status = gtk_statusbar_new ();
-  gtk_box_pack_end (GTK_BOX (window->main_box), window->status, FALSE, FALSE, 0);
-  if (window->statusbar_visible)
-    {
-      GAction *action = g_action_map_lookup_action (G_ACTION_MAP (window),
-                                                    "view-statusbar");
-      g_action_change_state (action, g_variant_new_boolean (TRUE));
-    }
-
-  window->progress = gtk_progress_bar_new ();
-  gtk_box_pack_end (GTK_BOX (window->status), window->progress, FALSE, FALSE, 0);
-
   /* retrieve the document font size */
   font_name = g_settings_get_string (window->desktop_settings, DOCUMENT_FONT_KEY);
   gdict_window_set_defbox_font (window, font_name);
@@ -1864,6 +1683,15 @@ gdict_window_class_init (GdictWindowClass *klass)
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
   GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
+  gtk_widget_class_set_template_from_resource (widget_class,
+                                               "/org/gnome/Dictionary/gdict-app-window.ui");
+
+  gtk_widget_class_bind_template_child (widget_class, GdictWindow, header_bar);
+  gtk_widget_class_bind_template_child (widget_class, GdictWindow, entry);
+  gtk_widget_class_bind_template_child (widget_class, GdictWindow, main_box);
+  gtk_widget_class_bind_template_child (widget_class, GdictWindow, spinner);
+  gtk_widget_class_bind_template_child (widget_class, GdictWindow, stack);
+
   gdict_window_properties[PROP_ACTION] =
     g_param_spec_enum ("action",
                        "Action",
@@ -1979,11 +1807,12 @@ gdict_window_init (GdictWindow *window)
   window->default_height = -1;
   window->is_maximized = FALSE;
   window->sidebar_visible = FALSE;
-  window->statusbar_visible = FALSE;
   window->sidebar_page = NULL;
   
   window->window_id = (gulong) time (NULL);
 
+  gtk_widget_init_template (GTK_WIDGET (window));
+
   /* we need to create the chooser widgets for the sidebar before
    * we set the construction properties
    */
@@ -1993,6 +1822,7 @@ gdict_window_init (GdictWindow *window)
 
 GtkWidget *
 gdict_window_new (GdictWindowAction  action,
+                  GtkApplication    *app,
 		  GdictSourceLoader *loader,
 		  const gchar       *source_name,
                   const gchar       *database_name,
@@ -2001,10 +1831,11 @@ gdict_window_new (GdictWindowAction  action,
 {
   GtkWidget *retval;
   GdictWindow *window;
-  
+
   g_return_val_if_fail (GDICT_IS_SOURCE_LOADER (loader), NULL);
   
   retval = g_object_new (GDICT_TYPE_WINDOW,
+                         "application", app,
   			 "action", action,
                          "source-loader", loader,
 			 "source-name", source_name,
